<?php
/*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2015 PrestaShop SA
*  @version  Release: $Revision: 1.2 $
*
*  International Registered Trademark & Property of PrestaShop SA
*/

/* Security */
if (!defined('_PS_VERSION_'))
	exit;

class GpnData extends PaymentModule
{
	private $_postErrors = array();

	private $_API;
    private $db;
	private $refundsActive = false;
	private $mustUseSSL = true;//if true then SSL must be enabled on server for plugin to work
	
	const PS_STATUS_SUCCESS = 8;
	const PS_STATUS_FAILIURE = 2;
	public function __construct()
	{
		$this->name = 'gpndata';
		$this->author = 'GPN DATA Dev Team';
		$this->tab = 'payments_gateways';
		$this->version = '1.0.0';

		parent::__construct();

		$this->displayName = $this->l('GPN DATA');
		$this->description = $this->l('Accept Credit card payments with GPN DATA International');

		/* Check if cURL is enabled */
		if (!is_callable('curl_exec'))
		{
			$this->warning = $this->l('cURL extension must be enabled on your server to use this module.');
			$this->active = false;
		}

		//if (!Configuration::get('PS_SSL_ENABLED') && $mustUseSSL)
		//{
		//    $this->warning = $this->l('Please enable SSL on your server to use this module.');
		//    $this->active = false;
		//}

		$this->initApi();
		/* Backward compatibility */
		if (_PS_VERSION_ < 1.5)
			require(_PS_MODULE_DIR_.'gpndata/backward_compatibility/backward.php');
	}
		
	
	private function initApi()
    {
        if (empty($this->_API)) {
            require_once(dirname(__FILE__) . '/library/gpndata.php');
            $this->_API = new GPNDataAPI(Configuration::get('GPNDATA_API_USER'),
                Configuration::get('GPNDATA_API_PASSWORD'),
                Configuration::get('GPNDATA_API_KEY'));
            $this->_API->transactionUrl = Configuration::get('GPNDATA_API_URL');
            $this->_API->prettyName = Configuration::get('PRETTY_NAME');
            $this->_API->iframeUrl = Configuration::get('IFRAME_URL');
            $this->_API->logoUrl = Configuration::get('LOGO_URL');
            $this->_API->contactUrl = Configuration::get('GPNDATA_CONTACT_URL');
            $this->_API->paymentPageUrl = Configuration::get('GPNDATA_PAYMENT_PAGE_URL');
            $this->db = new Database(_DB_PREFIX_, _DB_SERVER_, _DB_NAME_, _DB_USER_, _DB_PASSWD_);
        }
    }

	public function install()
	{
		$this->registerHook('displayMobileHeader');
		
		return Db::getInstance()->Execute('
		CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'gpndata` (
			
			`Id` BIGINT(20) NOT NULL AUTO_INCREMENT,
			`id_cart` INT(10) NOT NULL,
			`id_order` INT(10) NOT NULL,
			`id_transaction` VARCHAR(36) DEFAULT NULL,
			`merchant_transaction_id` VARCHAR(50) DEFAULT NULL,
			`amount` DECIMAL(20, 6) DEFAULT NULL,
			`transaction_status` VARCHAR(50) DEFAULT NULL,
			`description` VARCHAR(400) DEFAULT NULL,
			`curcode` VARCHAR(3) DEFAULT NULL,
			`date_add` DATETIME NOT NULL,
			`date_cancel` DATETIME DEFAULT NULL,
			`refund_reason` VARCHAR(100) DEFAULT NULL,
			`refunded_amount` DECIMAL(20, 6) DEFAULT NULL,
			`date_refund` DATETIME DEFAULT NULL,
			PRIMARY KEY (Id),
			UNIQUE INDEX Id (Id)
		) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;'. ' AUTO_INCREMENT = 1') && parent::install() 
		&& $this->registerHook('payment') 
		&& $this->registerHook('paymentReturn')
		&& $this->registerHook('adminOrder') 
		&& $this->registerHook('orderConfirmation');
	}

	public function uninstall()
	{
		Configuration::deleteByName('GPNDATA_API_KEY');
		Configuration::deleteByName('GPNDATA_API_USER');
		Configuration::deleteByName('GPNDATA_API_PASSWORD');
		Configuration::deleteByName('GPNDATA_ENVIROMENT_MODE');
		Configuration::deleteByName('GPNDATA_API_URL');
		Configuration::deleteByName('PRETTY_NAME');
		Configuration::deleteByName('IFRAME_URL');
		Configuration::deleteByName('LOGO_URL');
		Configuration::deleteByName('GPNDATA_CONTACT_URL');
		Configuration::deleteByName('GPNDATA_TRANSACTION_URL');
		Configuration::deleteByName('GPNDATA_PAYMENT_MERCHANT_NAME');
		Configuration::deleteByName('GPNDATA_PAYMENT_DESCRIPTION');
		Configuration::deleteByName('GPNDATA_PAYMENT_PAGE_URL');

		return parent::uninstall() && !$this->registerHook('payment')
        && !$this->unregisterHook('paymentReturn')
        && !$this->unregisterHook('adminOrder')
        && !$this->unregisterHook('orderConfirmation')
        && !$this->unregisterHook('displayPaymentTop');
	}

	private function getHomeUrl()
	{
		return Tools::getHttpHost(false).__PS_BASE_URI__;
	}

	public function getContent()
	{
		$html = '';

		if (Tools::isSubmit('submitConfigurationData') &&
			isset($_POST['gpndata_api_key']) && !empty($_POST['gpndata_api_key']) &&
			isset($_POST['gpndata_api_user']) && !empty($_POST['gpndata_api_user']) &&
			isset($_POST['gpndata_api_password']) && !empty($_POST['gpndata_api_password']) &&
			isset($_POST['gpndata_contact_url']) && !empty($_POST['gpndata_contact_url'])&&
			isset($_POST['pretty_name']) && !empty($_POST['pretty_name'])&&
			isset($_POST['iframe_url']) && !empty($_POST['iframe_url'])&&
			isset($_POST['gpndata_payment_page_url']) && !empty($_POST['gpndata_payment_page_url']))
		{
			Configuration::updateValue('FOOTERCUSTOM_NAME1',Tools::getValue('FOOTERCUSTOM_NAME1'));
			Configuration::updateValue('GPNDATA_PAYMENT_DESCRIPTION', pSQL(Tools::getValue('gpndata_payment_description')));
			Configuration::updateValue('GPNDATA_PAYMENT_PAGE_URL', pSQL(Tools::getValue('gpndata_payment_page_url')));
			Configuration::updateValue('GPNDATA_API_KEY', pSQL(Tools::getValue('gpndata_api_key')));
			Configuration::updateValue('GPNDATA_API_USER', pSQL(Tools::getValue('gpndata_api_user')));
			Configuration::updateValue('GPNDATA_API_PASSWORD', pSQL(Tools::getValue('gpndata_api_password')));
			Configuration::updateValue('GPNDATA_ENVIROMENT_MODE', pSQL(Tools::getValue('gpndata_environment_mode')));
			Configuration::updateValue('GPNDATA_PAYMENT_MERCHANT_NAME', pSQL(Tools::getValue('gpndata_payment_merchant_name')));

			Configuration::updateValue('GPNDATA_CONTACT_URL', pSQL(Tools::getValue('gpndata_contact_url')));
			Configuration::updateValue('GPNDATA_API_URL', pSQL(Tools::getValue('gpndata_api_url')));
			Configuration::updateValue('PRETTY_NAME', pSQL(Tools::getValue('pretty_name')));
			Configuration::updateValue('IFRAME_URL', pSQL(Tools::getValue('iframe_url')));
			Configuration::updateValue('LOGO_URL', pSQL(Tools::getValue('logo_url')));

			$html = '<div class="conf confirm">'.$this->l('Configuration updated successfully').'</div>';
		}
		else if (Tools::isSubmit('submitConfigurationData'))
			$html = '<div class="error">'.$this->l('Please fill the required fields').'</div>';

		$this->context->smarty->assign(array(
		'gpndata_form' => './index.php?tab=AdminModules&configure=gpndata&token='.Tools::getAdminTokenLite('AdminModules').'&tab_module='.$this->tab.'&module_name=gpndata',
		'gpndata_payment_description' => Configuration::get('GPNDATA_PAYMENT_DESCRIPTION'),
     	'gpndata_payment_page_url' => Configuration::get('GPNDATA_PAYMENT_PAGE_URL'),
		'gpndata_api_key' => Configuration::get('GPNDATA_API_KEY'),
		'gpndata_api_user' => Configuration::get('GPNDATA_API_USER'),
		'gpndata_api_password' => Configuration::get('GPNDATA_API_PASSWORD'),
		'gpndata_environment_mode' => (int)Configuration::get('GPNDATA_ENVIROMENT_MODE'),
		'gpndata_contact_url' => Configuration::get('GPNDATA_CONTACT_URL'),
		'gpndata_api_url' => Configuration::get('GPNDATA_API_URL'),
		'pretty_name' => Configuration::get('PRETTY_NAME'),
		'iframe_url' => Configuration::get('IFRAME_URL'),
		'logo_url' => Configuration::get('LOGO_URL'),
		'gpndata_notifications_url' => Tools::getHttpHost(true).__PS_BASE_URI__.'modules/gpndata/validation.php',
		'gpndata_ssl' => Configuration::get('PS_SSL_ENABLED'),
		'gpndata_payment_merchant_name' => Configuration::get('GPNDATA_PAYMENT_MERCHANT_NAME'),
		'gpndata_confirmation' => $html));

		return $this->display(__FILE__, 'tpl/admin.tpl');
	}

	public function hookDisplayMobileHeader()
	{
		return $this->hookHeader();
	}

	public function hookAdminOrder($params)
	{
		if (!$this->active)
			return false;
		$order = new Order((int)$params['id_order']);
		if ($order->module != $this->name)
			return false;
		if (!Validate::isLoadedObject($order))
			return false;


		$transaction = $this->getTransactionLog((int)$order->id_cart);
		
		$state = 'No information';
		$description = 'No information available';
		if(count($transaction)> 0)
		{
		    $last_transaction_state = $this->_API->CheckTransactionStatus($transaction[0]['merchant_transaction_id'], $transaction[0]['id_transaction']);
		    $state = $last_transaction_state->transaction->state;
		    $description = $last_transaction_state->transaction->description;
		}
		$this->context->smarty->assign(array(
		'gpndata_form' => './index.php?tab=AdminOrders&id_order='.(int)$order->id.'&vieworder&token='.Tools::getAdminTokenLite('AdminOrders'),
		'current_state' => $order->current_state,
		'historyRowCount' => count($transaction),
		'historyData' => $transaction,
		'currentStatus' => $state,
		'currentDescription' => $description));

		return $this->display(__FILE__, 'tpl/admin-order.tpl');
	}

    public function hookdisplayPaymentTop()
    {
        if (!$this->active || !Tools::getValue('gpnerror')) {
            return;
        }

        $errorMessage = Tools::getValue('gpnerror');

        $this->context->smarty->assign(array(
            'errorMessage' => $errorMessage,
            'module' => "gpndata"
        ));

        return $this->display(__FILE__, 'tpl/error.tpl');
    }

	/**
	 * Display a confirmation message after an order has been placed
	 *
	 * @param array Hook parameters
	 */
	public function hookOrderConfirmation($params)
	{

		if ($params['objOrder']->module != $this->name)
			return false;

		if ($params['objOrder'] && Validate::isLoadedObject($params['objOrder']) && isset($params['objOrder']->valid))
		{
			//$transactionLog = $this->getTransactionLogByOrderId($params['objOrder']->id);
			$description = 'Transaction Failed!'; //count($transactionLog) == 0 ? '' : $transactionLog[0]['description'];

			if (version_compare(_PS_VERSION_, '1.5', '>=') && isset($params['objOrder']->reference))
				$this->smarty->assign('gpndata_order',
					array('id' => $params['objOrder']->id, 
						'reference' => $params['objOrder']->reference, 
						'current_state' => $params['objOrder']->current_state, 
						'description' => $description));
			else
				$this->smarty->assign('gpndata_order', array('id' => $params['objOrder']->id, 'current_state' => $params['objOrder']->current_state , 'description' => $description));

			return $this->display(__FILE__, 'tpl/order-confirmation-gpn.tpl');
		}
	}

	public function hookPayment($params)
	{
$products = Context::getContext()->cart->getProducts();



foreach($products as $producte){
	$productlist[] =  $producte['name'].' x '.$producte['cart_quantity'];
	
}
 $description_item = implode(", ", $productlist);
 

		$merchanttransid = $this->context->cart->id.'-'.time().rand(100,999);
$customer = new Customer(intval($params['cart']->id_customer));
	$customerid = (string)$customer->id;
	$_API                 = new GPNDataAPI(
					Configuration::get('GPNDATA_API_USER'), Configuration::get('GPNDATA_API_PASSWORD'), Configuration::get('GPNDATA_API_KEY')
			);
	
	$address = new Address(intval($params['cart']->id_address_invoice));
	$customerphone = $address->phone;
	$customer_address = $address-> address1;

	$country = $_API->mapIso(CountryCore::getIsoById($address->id_country));			
	$states = State::getNameById($address->id_state);
	$postcode = $address->postcode;
	$city = $address->city;	
	$company = $this->_API->merchantName=Configuration::get('GPNDATA_PAYMENT_MERCHANT_NAME');
	$prettyname = Configuration::get('PRETTY_NAME');
	$iframeurl = Configuration::get('IFRAME_URL');
	$logourl = Configuration::get('LOGO_URL');
	$key = sha1(sha1($merchanttransid).'::'.Configuration::get('GPNDATA_API_KEY'));
	$status_ok = sha1(sha1($merchanttransid).'::'.Configuration::get('GPNDATA_API_KEY').'success');
	$status_failure = sha1(sha1($merchanttransid).'::'.Configuration::get('GPNDATA_API_KEY').'failure');
	$successurl = Tools::getHttpHost(true).__PS_BASE_URI__.'modules/gpndata/validation.php?&id_cart='.(int)$this->context->cart->id.'&transactionid='.$merchanttransid.'&status='.$status_ok.'&key='.$key;
	$failureurl = Tools::getHttpHost(true).__PS_BASE_URI__.'modules/gpndata/validation.php?&id_cart='.(int)$this->context->cart->id.'&transactionid='.$merchanttransid.'&status='.$status_failure.'&key='.$key;;
	$contacturl = Configuration::get('GPNDATA_CONTACT_URL');  // <-- for hyperlink in Payment Page	
		$currency = new Currency((int)$this->context->cart->id_currency);
		$cur = $currency->iso_code;
		
		
$merchant_prettyname =  $this->context->customer->lastname;
$this->smarty->assign('merchant_prettyname', $merchant_prettyname);

$success_url =  $successurl;
$this->smarty->assign('success_url', $success_url);
$failure_url =  $failureurl;
$this->smarty->assign('failure_url', $failure_url);
$company =  $company;
$this->smarty->assign('company', $company);
$prettyname =  $prettyname;
$this->smarty->assign('prettyname', $prettyname);


$iframeurl =  $iframeurl;
$this->smarty->assign('iframeurl', $iframeurl);


$logourl =  $logourl;
$this->smarty->assign('logourl', $logourl);

$curs =  $cur;
$this->smarty->assign('curs', $curs);

$item_description =  $description_item;
$this->smarty->assign('item_description', $item_description);

$order_id =  $merchanttransid;
$this->smarty->assign('order_id', $order_id);
$customer_id =  $customerid;
$this->smarty->assign('customer_id', $customer_id);
$order_email =  $this->context->customer->email;
$this->smarty->assign('order_email', $order_email);
$order_phone =  $customerphone;
$this->smarty->assign('order_phone', $order_phone);
$order_firstname =  $this->context->customer->firstname;
$this->smarty->assign('order_firstname', $order_firstname);
$order_lastname =  $this->context->customer->lastname;
$this->smarty->assign('order_lastname', $order_lastname);
$order_address =  $customer_address;
$this->smarty->assign('order_address', $order_address);
$order_zip =  $postcode;
$this->smarty->assign('order_zip', $order_zip);
$order_city =  $city;
$this->smarty->assign('order_city', $order_city);
$order_state =  $states;
$this->smarty->assign('order_state', $order_state);
$order_country =  $country;
$this->smarty->assign('order_country', $order_country);


		$this->smarty->assign('gpndata_ps_version', _PS_VERSION_);

		$currency = new Currency((int)$params['cart']->id_currency);
		if (!$this->active ||
		Configuration::get('GPNDATA_API_KEY')  == '' ||
		Configuration::get('GPNDATA_API_USER') == '' ||
		Configuration::get('GPNDATA_API_PASSWORD') == '' ||
		Configuration::get('GPNDATA_PAYMENT_PAGE_URL') == ''
		)
		return false;

		 /**
		* This function returns the total cart amount
		*
		* Possible values for $type:
		* Cart::ONLY_PRODUCTS
		* Cart::ONLY_DISCOUNTS
		* Cart::BOTH
		* Cart::BOTH_WITHOUT_SHIPPING
		* Cart::ONLY_SHIPPING
		* Cart::ONLY_WRAPPING
		* Cart::ONLY_PRODUCTS_WITHOUT_SHIPPING
		* Cart::ONLY_PHYSICAL_PRODUCTS_WITHOUT_SHIPPING
		*
		* @param bool $withTaxes With or without taxes
		* @param int $type Total type
		* @param bool $use_cache Allow using cache of the method CartRule::getContextualValue
		* @return float Order total
		*/

		$this->context->smarty->assign(array(
			'payment_error' => null,
            'token'         => Tools::getToken(false),
			'payment_url' => $this->context->link->getModuleLink($this->name, 'payment', array(), true).'&token='.Tools::getToken(false),
			'gpndata_payment_description' => Configuration::get('GPNDATA_PAYMENT_DESCRIPTION'),
			'gpndata_process_payment' => 1));
						return $this->display(__FILE__, 'tpl/payment_page.tpl');
						
	}

	public function HandleResponse($strData)
	{
		$message = 'OK';
		// create an XML object from strdata
		$xmldata = simplexml_load_string(stripslashes(urldecode($strData)));

		if (empty($xmldata))
		{
			unset($xmldata);
			$resstr = '
			<?xml version="1.0" encoding="utf-8" ?>
			<transaction>
			<result>ERROR</result >
			<gatetransid></gatetransid>
			<merchanttransid></merchanttransid>
			<errorcode></errorcode>
			<errormessage>No XML Data</errormessage>
			</transaction>';
			echo $resstr;
			die();
		}

		If (!$this->_API->validateNotificationChecksum($xmldata))
		{
			unset($xmldata);
			$resstr = '
			<?xml version="1.0" encoding="utf-8" ?>
			<transaction>
			<result>ERROR</result >
			<gatetransid></gatetransid>
			<merchanttransid></merchanttransid>
			<errorcode></errorcode>
			<errormessage>Invalid Checksum</errormessage>
			</transaction>';
			echo $resstr;
			die();
		}

			switch ($xmldata->apiCmd) {
			  case 850: //This is Transaction Notification
				$transaction_merchanttransid=$xmldata->merchanttransid;
				$transaction_gatetransid=$xmldata->gatetransid;
				$transaction_amount=$xmldata->amount;
				$transaction_curcode=$xmldata->curcode;
				$transaction_status=$xmldata->status;
				$transaction_state=$xmldata->state;
				$transaction_description=$xmldata->description;

				$message = $this->validateOrderFromNotification($transaction_merchanttransid, $transaction_state, $transaction_gatetransid, $transaction_amount,
				$transaction_description, $transaction_curcode);
				break;
			 case 850: //This is Transaction Notification
				$transaction_merchanttransid=$xmldata->merchanttransid;
				$transaction_gatetransid=$xmldata->gatetransid;
				$transaction_amount=$xmldata->amount;
				$transaction_curcode=$xmldata->curcode;
				$transaction_status=$xmldata->status;
				$transaction_state=$xmldata->state;
				$transaction_description=$xmldata->description;

				$message = $this->validateOrderFromNotification($transaction_merchanttransid, $transaction_state, $transaction_gatetransid, $transaction_amount,
				$transaction_description, $transaction_curcode);
				break;
			  case 880: //This is Chargeback Notification
				$chgbk_gatetransid=$xmldata->gatetransid;
				$chgbk_transref=$xmldata->transref;
				$chgbk_merchanttransid=$xmldata->merchanttransid;
				$chgbk_status=$xmldata->status;
				$chgbk_amount=$xmldata->amount;
				$chgbk_currency=$xmldata->currency;
				$chgbk_chbDate=$xmldata->chbDate;
				$chgbk_reasonCode=$xmldata->reasonCode;
				$chgbk_reasonText=$xmldata->reasonText;
				$chgbk_chbRef=$xmldata->chbRef;
				$message = $this->validateOrderFromNotification($chgbk_merchanttransid, $chgbk_status, $chgbk_gatetransid, $chgbk_amount,
				$chgbk_reasonText.'. Reason code: '.$chgbk_reasonCode.'\n ARN:'.$xmldata->arn, $chgbk_currency);
				break;
			  default:
					unset($xmldata);

		}
		//$message
			// The following logic will respond to the Notification so that we will know
			// it was received and reset the $xmldata object.
			$resstr = '
			<?xml version="1.0" encoding="utf-8" ?>
			<transaction>
				<result>OK</result>
				<plugin-message>'.$message.'</plugin-message>
				<gatetransid>'.$xmldata->gatetransid.'</gatetransid>
			</transaction>';
			echo $resstr;
			unset($xmldata);
	}

	public function validateOrderTransaction($id_cart, $transactionId, $status, $key)
	{
		$cart = $this->context->cart;
		$new_status = 8;
		$valid_key = sha1(sha1($transactionId).'::'.Configuration::get('GPNDATA_API_KEY'));
		if($valid_key == $key)
		{
			
			$status_ok = sha1(sha1($transactionId).'::'.Configuration::get('GPNDATA_API_KEY').'success');
			$status_failure = sha1(sha1($transactionId).'::'.Configuration::get('GPNDATA_API_KEY').'failure');
			
			if($status == $status_ok)
				$new_status = 2; //by default statuses prestashop Payment Success
				
			if($status == $status_failure)
				$new_status = 8; //by default statuses prestashop Payment Failed
		}
		else
		{
			$new_status = 8;
		}
		
	
		if(Validate::isLoadedObject($cart))
		{
			$total = (float)($cart->getOrderTotal());//getOrderTotal(true, Cart::BOTH));
			$this->validateOrder((int)$cart->id, (int)$new_status, (float)$total, $this->displayName, null, array(), null, false, $cart->secure_key);
			$this->addTransactionId($this->currentOrder, $transactionId);
				
		if (_PS_VERSION_ < 1.5)
			$redirect = __PS_BASE_URI__.'order-confirmation.php?id_cart='.(int)$this->context->cart->id.'&id_module='.(int)$this->id.'&id_order='.(int)$this->currentOrder.'&key='.$this->context->customer->secure_key;
		else
			$redirect = __PS_BASE_URI__.'index.php?controller=order-confirmation&id_cart='.(int)$this->context->cart->id.'&id_module='.(int)$this->id.'&id_order='.(int)$this->currentOrder.'&key='.$this->context->customer->secure_key;
				
			header('Location: '.$redirect);
			exit;
		}
		else
		{
			$id_order = Order::getOrderByCartId($id_cart);
			$order = new Order($id_order);
			$customer = new Customer($order->id_customer);
		
			if (_PS_VERSION_ < 1.5)
				$redirect = __PS_BASE_URI__.'order-confirmation.php?id_cart='.(int)$id_cart.'&id_module='.(int)$this->id.'&id_order='.(int)$id_order.'&key='.$customer->secure_key;
			else
				$redirect =__PS_BASE_URI__.'index.php?controller=order-confirmation&id_cart='.(int)$id_cart.'&id_module='.(int)$this->id.'&id_order='.(int)$id_order.'&key='.$customer->secure_key;
		
			header('Location: '.$redirect);
			exit;
		}		
	}

	public function validateOrderFromNotification($merchanttransid, $state, $transaction_gatetransid, $amount, $description, $curcode)
	{
		$message = 'OK';
		$val = explode('-', $merchanttransid);
		$id_cart = $val[0];


		$idOrder =	Order::getOrderByCartId($id_cart);
		$order = new Order($idOrder);
		$orderReference = $order->reference;
		$order_validated = false;
		if (Validate::isLoadedObject($order))
			$order_validated = true;

		if(!$order_validated && ($state == 'CAPTURED' || $state == 'DECLINED'))
		{
			$cart = new Cart($id_cart);
			if(Validate::isLoadedObject($cart))
			{
				$current_state = -1;
				if($state == 'CAPTURED')
					$current_state = 2;

				if($state == 'DECLINED')
					$current_state = 8;

				$this->validateOrder((int)$cart->id, (int)$current_state, (float)$amount, $this->displayName, null, array(), null, false, $cart->secure_key);

				if (version_compare(_PS_VERSION_, '1.5', '>='))
				{
					$new_order = new Order((int)$this->currentOrder);
					if (Validate::isLoadedObject($new_order))
					{
						$this->addTransactionId($this->currentOrder, $merchanttransid);
						$this->insertTransactionLog(array('id_cart' => $cart->id,
											'id_order' => $new_order->id,
											'transaction_status' => $state,
											'amount' => $amount,
											'merchant_transaction_id' => $merchanttransid,
											'curcode' => $curcode,
											'id_transaction' => $transaction_gatetransid,
											'description' => $description,
											'date_add' => date('Y-m-d H:i:s')));

					}
				}
				$message = "Order was successfully created with status $current_state. Status 8 = declined, status 2 approved!";

				if($this->currentOrder >0)//since on pending status order cant be created, order id is = 0, now that it has been created update the transaction log table
					$this->updateTransactionLogOrderId($merchanttransid, $this->currentOrder);
			}
			else
			{
				$message = 'Error - Invalid cart Id';
			}
		}
		else
		{
			$this->insertTransactionLog(array('id_cart' => $id_cart,
											'id_order' => (int)$idOrder,
											'transaction_status' => $state,
											'amount' => $amount,
											'merchant_transaction_id' => $merchanttransid,
											'id_transaction' => $transaction_gatetransid,
											'description' => $description,
											'curcode' => $curcode,
											'date_add' => date('Y-m-d H:i:s')));

			$message = 'Insert or update transaction state table OK!';
		}

		return $message;
	}


	public function addTransactionId($id_order, $id_transaction)
	{
		if (version_compare(_PS_VERSION_, '1.5', '>='))
		{
			$new_order = new Order((int)$id_order);
			if (Validate::isLoadedObject($new_order))
			{
				$payment = $new_order->getOrderPaymentCollection();
				if (isset($payment[0]))
				{
					$payment[0]->transaction_id = pSQL($id_transaction);
					$payment[0]->save();
				}
			}
		}
	}

	#region Database

	private function insertTransactionLog($params)
	{

		$status = strtolower($params['transaction_status']);
		$status_count = $this->getTransactionLogByIdCartAndStatus((int)$params['id_cart'], $params['transaction_status']);
		$current_status_count = count($status_count);
		if($status == 'pending')
		{
			if($current_status_count == 0)
			{
			   return Db::getInstance()->insert('gpndata', $params);
			}
		}
		else
		{
			 return Db::getInstance()->insert('gpndata', $params);
		}
	}

	private function updateTransactionLogOrderId($transactionId, $id_order)
	{
		return Db::getInstance()->update('gpndata', array('id_order' =>(int)$id_order), 'merchant_transaction_id  = "'.$transactionId.'"');
		//return Db::getInstance()->execute('UPDATE `'._DB_PREFIX_.'gpndata` SET `id_order` ="'.(int)$id_order.'" WHERE `merchant_transaction_id` = "'.$transactionId.'"');
	}

	private function getTransactionLog($id_cart)
	{
		return Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'gpndata` WHERE date_refund IS NULL AND `id_cart` = '.(int)$id_cart);
	}

	private function getTransactionLogByOrderId($id_order)
	{
		return Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'gpndata` WHERE `id_order` = '.(int)$id_order);
	}

	private function getTransactionLogByIdCartAndStatus($id_cart, $status)
	{
		return Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'gpndata` WHERE `id_cart` = '.(int)$id_cart. ' AND `transaction_status` = "'.$status.'"');
	}

	#end region
}
