<?php

class Database
{

	public function __construct($table_prefix, $host = '', $db_name = '', $user = '', $pass = ''){

		try {
			$this->db           = new PDO('mysql:host=' . $host . ';dbname=' . $db_name, $user, $pass);
			$this->table_prefix = $table_prefix;
		}
		catch(PDOException $e){
			// If something goes wrong, PDO throws an exception with a nice error message.
			echo $e->getMessage();
		}
	}

	public function createTables(){

		try {
			$this->db->query(
					"
				CREATE TABLE IF NOT EXISTS `" . $this->table_prefix . "gpndata_orders` (
					  Id INT(11) NOT NULL AUTO_INCREMENT,
					  order_id INT(11) NOT NULL,
					  order_ref CHAR(50) DEFAULT NULL,
					  merchant_trans_id CHAR(50) NOT NULL,
					  gate_trans_id CHAR(50) NOT NULL,
					  curcode CHAR(3) NOT NULL,
					  total_amount DECIMAL(10, 2) NOT NULL,
					  merchantspecific1 CHAR(50) DEFAULT NULL,
					  merchantspecific2 CHAR(50) DEFAULT NULL,
					  merchantspecific3 CHAR(50) DEFAULT NULL,
					  merchantspecific4 CHAR(50) DEFAULT NULL,
					  is_refund_pending INT(11) DEFAULT 0,
					  is_captured INT(11) DEFAULT 0,
					  is_voided INT(11) DEFAULT 0,
					  is_authorized INT(11) DEFAULT 0,
					  is_declined INT(11) DEFAULT 0,
					  date_added DATETIME NOT NULL,
					  date_modified DATETIME DEFAULT NULL,
					  json_data TEXT DEFAULT NULL,
					  PRIMARY KEY (Id)
				) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;"
			);

			$this->db->query(
					"
			CREATE TABLE IF NOT EXISTS `" . $this->table_prefix . "gpndata_orders_transactions` (
				  order_transaction_id INT(11) NOT NULL AUTO_INCREMENT,
				  order_id INT(11) NOT NULL,
				  status VARCHAR(50) NOT NULL,
				  state VARCHAR(50) NOT NULL,
				  description VARCHAR(255) DEFAULT NULL,
				  additional_text VARCHAR(255) DEFAULT NULL,
				  errormessage VARCHAR(255) DEFAULT NULL,
				  amount DECIMAL(16, 2) NOT NULL,
				  date_added DATETIME DEFAULT NULL,
				  PRIMARY KEY (order_transaction_id)
			) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;"
			);

		}
		catch(Exception $e){
			echo $e->getMessage();

			return false;
		}

		return true;
	}

	public function dropTables(){
		try {
			$this->db->query("DROP TABLE IF EXISTS `" . $this->table_prefix . "gpndata_orders`;");
			$this->db->query("DROP TABLE IF EXISTS `" . $this->table_prefix . "gpndata_orders_transactions`;");
		}
		catch(Exception $exception){
			$this->throwDbError($exception->getMessage());

			return false;
		}

		return true;
	}

	/*Mandatory fields simple example. Extened the array with additional table fields if there is a need
	$orderData = array('order_id'=>'','merchant_trans_id'=>'', 'gate_trans_id'=>'', 'date_added' => 'now()', 'curcode' => '', 'total_amount' => ''); */
	function addOrder($order){

		$query = "INSERT INTO `" . $this->table_prefix . "gpndata_orders` SET " . $this->bindFields($order) . ", date_added = now()";
		$stmt  = $this->db->prepare($query);
		$stmt->execute($order) or $this->throwDbError($stmt->errorInfo());
	}

	public function updateOrder($order){

		$query = "UPDATE `" .
		         $this->table_prefix .
		         "gpndata_orders` SET " .
		         $this->bindFields($order, 'order_id') .
		         ", date_modified = now() " .
		         " WHERE `order_id` = :order_id";
		$stmt  = $this->db->prepare($query);
		$stmt->execute($order) or $this->throwDbError($stmt->errorInfo());
	}

	public function getOrder($order_id, $fetch_transactions = false){

		$stmt = $this->db->prepare("SELECT * FROM `" . $this->table_prefix . "gpndata_orders` WHERE `order_id` = :order_id LIMIT 1");
		$stmt->bindParam(':order_id', $order_id, PDO::PARAM_INT);
		$stmt->execute() or $this->throwDbError($stmt->errorInfo());
		if($stmt->rowCount() > 0){
			$order = $this->getData($stmt);
			if($fetch_transactions){

				$order['transactions'] = $this->getTransactions($order['order_id']);
			}

			return $order;
		}
		else{
			return false;
		}
	}

	public function orderExists($order_id){

		$stmt = $this->db->prepare("SELECT * FROM `" . $this->table_prefix . "gpndata_orders` WHERE `order_id` = :order_id LIMIT 1");
		$stmt->bindParam(':order_id', $order_id, PDO::PARAM_INT);
		$stmt->execute() or $this->throwDbError($stmt->errorInfo());

		return $stmt->rowCount() > 0;
	}

	//Mandatory fields simple example. Extened the array with additional table fields if there is a need
	//$orderData = array('gpndata_order_id'=>'','status'=>'', 'state'=>'', 'description' => '', 'date_added' => 'now()', 'amount' => '');
	public function addTransaction($orderTransaction, $debug = false){

		if($this->isTransactionDataInsertNeeded(
				$orderTransaction['order_id']
		)
		){//there are some special cases where status of table can we updated wrong, check function for more info
			$query = "INSERT INTO `" . $this->table_prefix . "gpndata_orders_transactions` SET " . $this->bindFields($orderTransaction) . ", date_added = now()";
			$stmt  = $this->db->prepare($query);
			$stmt->execute($orderTransaction) or $this->throwDbError($stmt->errorInfo());
		}
	}

	public function updateTransaction($orderTransaction){

		$query = "UPDATE `" .
		         $this->table_prefix .
		         "gpndata_orders_transactions` SET " .
		         $this->bindFields($orderTransaction, 'order_id') .
		         " WHERE `order_id` = :order_id";
		$stmt  = $this->db->prepare($query);
		$stmt->execute($orderTransaction) or $this->throwDbError($stmt->errorInfo());
	}

	private function getTransactions($id){

		$stmt = $this->db->prepare("SELECT * FROM `" . $this->table_prefix . "gpndata_orders_transactions` WHERE `order_id` = :id");
		$stmt->bindParam(":id", $id, PDO::PARAM_INT);
		$stmt->execute() or $this->throwDbError($stmt->errorInfo());
		if($stmt->rowCount() > 0){
			return $this->getData($stmt, true);
		}
		else{
			return false;
		}
	}

	public function getLastTransaction($order_id){

		$stmt = $this->db->prepare(
				"SELECT * FROM `" . $this->table_prefix . "gpndata_orders_transactions` WHERE `order_id` = :order_id ORDER BY order_transaction_id DESC LIMIT 1"
		);
		$stmt->bindParam(":order_id", $order_id, PDO::PARAM_INT);
		$stmt->execute();
		if($stmt->rowCount() > 0){
			return $this->getData($stmt);
		}
		else{
			return false;
		}
	}

	public function createUniqueKey($order_id){

		$order = $this->getOrder($order_id, false);

		return sha1($order_id . $order['gate_trans_id'] . $order['date_added']);
	}

	function bindFields($fields, $ignore_field = ''){

		end($fields);
		$lastField  = key($fields);
		$bindString = ' ';
		foreach($fields as $field => $data){

			if(strlen($ignore_field) > 0){
				if(strtolower($field) == strtolower($ignore_field)){
					continue;
				}
			}

			$bindString .= $field . '=:' . $field;
			$bindString .= ($field === $lastField ? ' ' : ',');
		}

		return $bindString;
	}

	private function getData($stmt, $fetchAll = false){

		if($fetchAll){
			$row = $stmt->fetchAll(PDO::FETCH_ASSOC);
			$stmt->closeCursor();
		}
		else{
			$row = $stmt->fetch(PDO::FETCH_ASSOC);
		}

		return $row;
	}

	private function throwDbError($errorInfo){

		$msg = '<pre>' . print_r($errorInfo) . '</pre>';
		throw new Exception($msg);
	}

	private function isTransactionDataInsertNeeded($order_id){

		try {
			$transaction = $this->getLastTransaction($order_id);
			if($transaction['status'] === TransactionStatus::CANCELED){
				return false;
			}
		}
		catch(Exception $exception){
			echo 'Error occured while trying to check if data insert is needed. Error details: ' + $exception->getMessage();

			return false;
		}

		return true;
	}

	public function setDatabase($db){
		$this->db = $db;
	}

	//Properties
	protected $db;
	//table prefix if any
	protected $table_prefix;

}

class ResponseStatusCode
{

	const NO_CONNECTION              = 0;
	const OK                         = 200;
	const CREATED                    = 201;
	const ACCEPTED                   = 202;
	const SERVICE_UNAVAILABLE        = 503;
	const GATEWAY_TIME_OUT           = 504;
	const HTTP_VERSION_NOT_SUPPORTED = 505;
}

class TransactionStatus
{

	const AUTHORIZED      = 'AUTHORIZED';
	const SUCCESS         = 'SUCCESS';
	const CAPTURED        = 'CAPTURED';
	const DECLINED        = 'DECLINED';
	const PENDING         = 'PENDING';
	const CANCELED        = 'CANCELED';
	const CREDITED        = 'CREDITED';
	const ERROR           = 'ERROR';
	const REVERSED        = 'REVERSED';
	const REFUNDED        = 'REFUNDED';
	const REFUND_PENDING  = 'REFUND PENDING';
	const CHARGEBACK      = 'CHARGEBACK';
	const REQUEST_PENDING = 'RequestPending';
}

class APICommands
{

	const StartCreditCardCharge            = 700;
	const RequestCaptureAuth               = 701;
	const RequestCancelAuth                = 702;
	const ThreeDSTransactionContinuation   = 705;
	const GetTransactionDetails            = 708;
	const CheckTransactionStatus           = 709;
	const StartCreditCardChargePaymentPage = 710;
	const CreditRequest                    = 720;
	const UpdateRebillInstructions         = 755;
	const ManualRebillRequest              = 756;
	const RequestRefund                    = 760;
}

class NotificationResponses
{

	const TransactionStateNotification = 850;
	const RefundNotification           = 860;
	const RebillNotification           = 870;
	const DisputeNotification          = 880;
}

class GPNDataAPI
{

	public function __construct($APIUser, $APIPassword, $APIKey, $debug = false){

		if(!extension_loaded("curl")){
			$this->throwException('Curl is not installed and this is not a supported scenario for this library!', true, __LINE__);
		}

		$this->apiUser     = $APIUser;
		$this->apiPassword = $APIPassword;
		$this->apiKey      = $APIKey;
		$this->validator   = new xmlValidator();
		$this->debug_mode  = $debug;
	}


	#region Exception messages - left here for translation in the future...

	private $EXCEPTION_MESSAGE_ERROR_MISSING_TRANSACTION_URL = 'Please set the transaction URL!';

	#end region


	/** Creates SimpleXMLElement for request  */
	public function initRequest($cmd, $successurl = '', $failureurl = '', $contacturl = ''){

		$xmlRequestObject = new SimpleXMLElement('<?xml version="1.0" encoding="utf-8" ?><transaction></transaction>');
		$xmlRequestObject->addChild('apiUser', $this->apiUser);
		$xmlRequestObject->addChild('apiPassword', $this->apiPassword);
		$xmlRequestObject->addChild('apiCmd', $cmd);
		if($cmd == APICommands::StartCreditCardChargePaymentPage){

			$xmlRequestObject->addChild('successurl', htmlspecialchars($successurl));
			$xmlRequestObject->addChild('failureurl', htmlspecialchars($failureurl));
			$xmlRequestObject->addChild('contacturl', htmlspecialchars($contacturl));

			if(strlen($this->merchantName) <= 100 && !empty($this->merchantName)){
				$xmlRequestObject->addChild('merchantname', $this->merchantName);
			}
		}

		return $xmlRequestObject;
	}


	public function constructTransactionDetails(
			$merchanttransid,
			$amount,
			$curcode,
			$description,
			$merchantspecific1 = '',
			$merchantspecific2 = '',
			$merchantspecific3 = '',
			$merchantspecific4 = '',
			$merchantspecific5 = ''
	){

		$transactionDetails = new SimpleXMLElement('<transaction></transaction>');
		$transactionDetails->addChild('merchanttransid', $merchanttransid);
		$transactionDetails->addChild('amount', $this->formatAmount($amount));
		$transactionDetails->addChild('curcode', $curcode);
		$transactionDetails->addChild('description', $description);
		if(!empty($merchantspecific1)){
			$transactionDetails->addChild('merchantspecific1', $merchantspecific1);
		}
		if(!empty($merchantspecific2)){
			$transactionDetails->addChild('merchantspecific2', $merchantspecific2);
		}
		if(!empty($merchantspecific3)){
			$transactionDetails->addChild('merchantspecific3', $merchantspecific3);
		}
		if(!empty($merchantspecific4)){
			$transactionDetails->addChild('merchantspecific4', $merchantspecific4);
		}
		if(!empty($merchantspecific5)){
			$transactionDetails->addChild('merchantspecific5', $merchantspecific5);
		}

		if($this->validator->validateXmlFields($transactionDetails, $this->debug_mode)){
			return $transactionDetails;
		}
		else{
			$this->throwException('Construction of TransactionDetails failed because of validation errors!', true, __LINE__);
		}
	}

	/** Construct customer details xml element
	 * Returns SimpleXMLElement of customer details.
	 * @param $firstname      = First name of the Customer.
	 * @param $lastname       = Last name of the Customer.
	 * @param $email          = Email of the Customer.
	 * @param $ipaddress      = Full IP address of the customer that is being used for this transaction.
	 * @param $countryiso     = 3-character ISO country code of the Customer. Full list is contanied in property $iso_map.
	 * @param $stateregioniso = 2-character ISO state/region/province code of the Customer. Full list of codes can be obtained from the website inside your
	 *                        merchant account. Note: $stateregioniso is optional if NOT from USA or CAN!
	 * @param $zippostal      = Customer�s Zip code or postal code.
	 * @param $city           = Customer�s City
	 * @param $address1       = Customer�s address line 1
	 * @param $address2       = Customer�s Address line 2 (if needed)
	 * @param $phone1phone    = Customer�s phone number.
	 * @param $accountid      = Customer unique account id used by the merchant to identify the customer. If you do not use customer IDs, you may use the email
	 *                        address.
	 */
	public function constructCustomerDetails(
			$firstname,
			$lastname,
			$email,
			$ipaddress,
			$countryiso = '',
			$stateregioniso = '',
			$zippostal = '',
			$city = '',
			$address1 = '',
			$address2 = '',
			$phone1phone = '',
			$accountid = ''
	){

		if(($countryiso == 'CAN' || $countryiso == 'USA') && empty($stateregioniso)){
			$this->throwException('If countryiso is United states or countryiso is Canada stateregioniso must be specified!!', true, __LINE__);

			return null;
		}

		$customerDetails = new SimpleXMLElement('<customer></customer>');
		$customerDetails->addChild('firstname', $firstname);
		$customerDetails->addChild('lastname', $lastname);
		$customerDetails->addChild('email', $email);
		$customerDetails->addChild('ipaddress', $ipaddress);

		if(!empty($countryiso)){
			$customerDetails->addChild('countryiso', $countryiso);
		}
		if(!empty($stateregioniso)){
			$customerDetails->addChild('stateregioniso', $stateregioniso);
		}
		if(!empty($zippostal)){
			$customerDetails->addChild('zippostal', $zippostal);
		}
		if(!empty($city)){
			$customerDetails->addChild('city', $city);
		}
		if(!empty($address1)){
			$customerDetails->addChild('address1', $address1);
		}
		if(!empty($address2)){
			$customerDetails->addChild('address2', $address2);
		}
		if(!empty($phone1phone)){
			$customerDetails->addChild('phone1phone', $phone1phone);
		}
		if(!empty($accountid)){
			$customerDetails->addChild('accountid', $accountid);
		}

		if($this->validator->validateXmlFields($customerDetails, $this->debug_mode)){
			return $customerDetails;
		}
		else{
			$this->throwException('Construction of CustomerDetails failed because of validation errors!', true, __LINE__);
		}

	}

	/** Construct customer details xml element
	 * Returns SimpleXMLElement of product details.
	 * @param $products_arr        = Array of products.
	 * @param $mapping_field_array = This array needs to contain key/value pair of your and GPNDATA column names paired
	 *                             Here is an example mapping array;
	 *                             $mapping_field_array =
	 *                             array('your_NAME_field_of_product' => 'name', //Name of the product or service
	 *                             'your_DESCRIPTION_field_of_product' => 'description', //Description of the product or service
	 *                             'your_PRICE_field_of_product' => 'price', //Unit price of this product or service (seenote on �transaction->amount� for details
	 *                             'your_AMOUNT_field_of_product' => 'amount', //Quantity of this product or service
	 *                             );
	 *                             Where columns on left are your COLUMN NAMES as KEY.
	 */
	public function constructProductDetailsPaymentPage($products_arr, $mapping_field_array){

		$products_count       = count($products_arr);
		$shoppingCartProducts = '';
		if($products_count == 0){
			$this->throwException('Product count must be greater then zero!', true, __LINE__);
		}

		$mapping_field_array_count = count($mapping_field_array);

		if($mapping_field_array_count == 0 || $mapping_field_array_count == 3){
			$this->throwException('Param $mapping_field_array must have len equal to three!', true, __LINE__);
		}

		$productDetails = new SimpleXMLElement('<products></products>');

		try {
			$name        = $this->getProductColumnValue('name', $mapping_field_array);
			$description = $this->getProductColumnValue('description', $mapping_field_array);
			$price       = $this->getProductColumnValue('price', $mapping_field_array);
			$amount      = $this->getProductColumnValue('amount', $mapping_field_array);
			foreach($products_arr as $key => $val){
				$this->appendSimpleXMLElement($productDetails, $this->constructProductPaymentPage($val[$name], $val[$description], $val[$price], $val[$amount]));
			}
		}
		catch(Exception $e){
			$this->throwException('Caught exception: ' . $e->getMessage(), true, __LINE__);
		}

		return $productDetails;
	}

	public function getProductColumnValue($column_name, $product_columns){

		reset($product_columns);
		while(list($key, $value) = each($product_columns)){
			if($key == $column_name){
				return $value;
			}
		}
	}

	public function constructProductPaymentPage($name, $description, $price, $quantity, &$shoppingCartProducts = ''){

		$product = '<product>
					<name><![CDATA[' . $name . ']]></name>
					<description><![CDATA[' . $description . ']]></description>
					<price><![CDATA[' . $this->formatAmount($price) . ']]></price>
					<amount><![CDATA[' . $this->formatAmount($quantity) . ']]></amount>
				 </product>';

		return new SimpleXMLElement($product);
	}

	//to be implemented!
	public function startCreditCardCharge($transactionDetails, $customerDetails, $rebillDetails = ''){

		try {
			/*
				$_XML = $this->initRequest(APICommands::StartCreditCardChargePaymentPage);
				$_XML = $_XML->addChild($transactionDetails);
				if(!empty($rebillDetails))
				{

				}
				*/
		}
		catch(Exception $e){
			$this->throwException('GPN DATA API - Caught exception: ' . $e->getMessage(), "\n", true, __LINE__);
		}
	}

	public function startCreditCardChargePaymentPage($transactionDetails, $customerDetails, $productDetails, $successUrl, $failureUrl, $contactUrl){

		$APIRequest  = null;
		$transaction = null;

		try {
			if(empty($this->paymentPageUrl)){
				$this->throwException('Before running this command please set the paymentPageUrl property!', true, __LINE__);
			}

			$transaction = $this->initRequest(APICommands::StartCreditCardChargePaymentPage, $successUrl, $failureUrl, $contactUrl);

			$this->appendSimpleXMLElement($transaction, $transactionDetails);
			$this->appendSimpleXMLElement($transaction, $customerDetails);
			$this->appendSimpleXMLElement($transaction, $productDetails);

			$transaction->addChild(
					'checksum',
					$this->calculateCheckSum(
							APICommands::StartCreditCardChargePaymentPage,
							$transactionDetails->amount,
							$transactionDetails->merchanttransid,
							null,
							$transactionDetails->curcode
					)
			);
			$APIRequest = new APIRequest();
			$APIRequest->executeRequest($transaction->asXML(), $this->paymentPageUrl);

			return $APIRequest->response;
		}
		catch(Exception $e){
			$this->throwException('GPN DATA API - Caught exception: ' . $e->getMessage(), "\n", true, __LINE__);
		}

		return null;
	}

	public function getAPIRequestXML($transactionDetails, $customerDetails, $productDetails, $successUrl, $failureUrl, $contactUrl, $cmd){

		$APIRequest  = null;
		$transaction = null;

		try {

			$transaction = $this->initRequest(APICommands::StartCreditCardChargePaymentPage, $successUrl, $failureUrl, $contactUrl);
			switch(intval($cmd)){
				case APICommands::StartCreditCardChargePaymentPage:

					if(empty($this->paymentPageUrl)){
						$this->throwException('Before running this command please set the paymentPageUrl property!', true, __LINE__);
					}

					$this->appendSimpleXMLElement($transaction, $transactionDetails);
					$this->appendSimpleXMLElement($transaction, $customerDetails);
					$this->appendSimpleXMLElement($transaction, $productDetails);
					$transaction->addChild(
							'checksum',
							$this->calculateCheckSum(
									APICommands::StartCreditCardChargePaymentPage,
									$transactionDetails->amount,
									$transactionDetails->merchanttransid,
									null,
									$transactionDetails->curcode
							)
					);
				break;
				default:
			}

			return $transaction->asXML();
		}
		catch(Exception $e){
			$this->throwException('GPN DATA API - Caught exception: ' . $e->getMessage(), "\n", true, __LINE__);
		}

		return null;
	}

	public function requestCaptureAuthorization($gatetransid, $amount){

		try {
			if(empty($this->transactionUrl)){
				$this->throwException('Please set the transaction URL!', true);
			}

			$_XML = $this->initRequest(APICommands::RequestCaptureAuth);
			$_XML->addChild('gatetransid', $gatetransid);
			$_XML->addChild('amount', $this->formatAmount($amount));


			$_XML->addChild('checksum', $this->calculateCheckSum(APICommands::RequestCaptureAuth, $amount, null, $gatetransid));

			$APIRequest = new APIRequest();
			$APIRequest->executeRequest($_XML->asXML(), $this->transactionUrl);

			return $APIRequest;
		}
		catch(Exception $e){
			$this->throwException('GPN DATA API - Caught exception: ' . $e->getMessage(), "\n", true, __LINE__);
		}

		return null;
	}

	public function requestCancelAuthorization($gatetransid){

		try {
			if(empty($this->transactionUrl)){
				$this->throwException('Please set the transaction URL!', true);
			}

			$_XML = $this->initRequest(APICommands::RequestCancelAuth);
			$_XML->addChild('gatetransid', $gatetransid);


			$_XML->addChild('checksum', $this->calculateCheckSum(APICommands::RequestCancelAuth, null, null, $gatetransid));

			$APIRequest = new APIRequest();
			$APIRequest->executeRequest($_XML->asXML(), $this->transactionUrl);

			return $APIRequest;
		}
		catch(Exception $e){
			$this->throwException('GPN DATA API - Caught exception: ' . $e->getMessage(), "\n", true, __LINE__);
		}

		return null;
	}

	public function refundTransaction($gatetransid, $amount, $reason){

		try {
			if(empty($this->transactionUrl)){
				$this->throwException('Please set the transaction URL!', true);
			}

			$_XML = $this->initRequest(APICommands::RequestRefund);
			$_XML->addChild('gatetransid', $gatetransid);
			$_XML->addChild('amount', $this->formatAmount($amount));
			$_XML->addChild('reason', $reason);

			$_XML->addChild('checksum', $this->calculateCheckSum(APICommands::RequestRefund, $amount, null, $gatetransid));

			$APIRequest = new APIRequest();
			$APIRequest->executeRequest($_XML->asXML(), $this->transactionUrl);

			return $APIRequest;
		}
		catch(Exception $e){
			$this->throwException('GPN DATA API - Caught exception: ' . $e->getMessage(), "\n", true, __LINE__);
		}

		return null;
	}

	/** 709 � Check transaction status
	 * Returns current transaction state. Optionally sends Notification 850.
	 * @param $merchanttransid = Unique merchant transaction id received in the request 700. Required if gatetransid not provided. Otherwise ignored.
	 * @param $gatetransid     = Unique merchant transaction id received in the request 700. Required if gatetransid not provided. Otherwise ignored.
	 * @param $notify          = YES: command will also trigger Notification 850 Other value: ignored.
	 */
	//($apiCmd, $amount, $merchanttransid = '', $gatetransid = '', $currency = '')
	public function checkTransactionStatus($merchanttransid, $gatetransid, $notify = false){
		try {
			$_XML = $this->initRequest(APICommands::CheckTransactionStatus);
			$_XML->addChild('merchanttransid', $merchanttransid);
			$_XML->addChild('gatetransid', $gatetransid);
			$_XML->addChild('notify', $notify ? 'YES' : '');
			$_XML->addChild('checksum', $this->calculateCheckSum(APICommands::CheckTransactionStatus, null, $merchanttransid, $gatetransid));
			if(empty($this->transactionUrl)){
				$this->throwException($this->EXCEPTION_MESSAGE_ERROR_MISSING_TRANSACTION_URL, true);
			}
			else{
				$APIRequest = new APIRequest();
				$APIRequest->executeRequest($_XML->asXML(), $this->transactionUrl);

				return $APIRequest->response;
			}
		}
		catch(Exception $e){
			$this->throwException('GPN DATA API - Caught exception: ' . $e->getMessage(), "\n", true, __LINE__);
		}

		return null;
	}


	/** 709 � Check transaction status
	 * Returns current transaction state. Optionally sends Notification 850.
	 * @param $merchanttransid = Unique merchant transaction id received in the request 700. Required if gatetransid not provided. Otherwise ignored.
	 * @param $gatetransid     = Unique merchant transaction id received in the request 700. Required if gatetransid not provided. Otherwise ignored.
	 * @param $notify          = YES: command will also trigger Notification 850 Other value: ignored.
	 */
	//($apiCmd, $amount, $merchanttransid = '', $gatetransid = '', $currency = '')
	public function getTransactionDetails($merchanttransid, $gatetransid){
		try {
			$_XML = $this->initRequest(APICommands::GetTransactionDetails);
			$_XML->addChild('merchanttransid', $merchanttransid);
			$_XML->addChild('gatetransid', $gatetransid);
			$_XML->addChild('checksum', $this->calculateCheckSum(APICommands::GetTransactionDetails, null, $merchanttransid, $gatetransid));
			if(empty($this->transactionUrl)){
				$this->throwException($this->EXCEPTION_MESSAGE_ERROR_MISSING_TRANSACTION_URL, true);
			}
			else{
				$APIRequest = new APIRequest();
				$APIRequest->executeRequest($_XML->asXML(), $this->transactionUrl);

				return $APIRequest->response;
			}
		}
		catch(Exception $e){
			$this->throwException('GPN DATA API - Caught exception: ' . $e->getMessage(), "\n", true, __LINE__);
		}

		return null;
	}

	public function calculateCheckSum($apiCmd, $amount, $merchanttransid = '', $gatetransid = '', $currency = ''){

		$checksum = '';
		switch($apiCmd){
			case APICommands::StartCreditCardCharge:

			break;
			case APICommands::StartCreditCardChargePaymentPage:
				if(empty($currency)){
					$this->throwException('Currency must be specified in order to calculate Checksum!', true);
				}
				$checksum = strtoupper(
						sha1(
								$this->apiUser .
								$this->apiPassword .
								(string)APICommands::StartCreditCardChargePaymentPage .
								$merchanttransid .
								$amount .
								$currency .
								$this->apiKey
						)
				);
			break;
			case APICommands::CheckTransactionStatus:
				$checksum = strtoupper(
						sha1($this->apiUser . $this->apiPassword . (string)APICommands::CheckTransactionStatus . $gatetransid . $merchanttransid . $this->apiKey)
				);
			break;
			case APICommands::GetTransactionDetails:
				$checksum = strtoupper(
						sha1($this->apiUser . $this->apiPassword . (string)APICommands::GetTransactionDetails . $gatetransid . $merchanttransid . $this->apiKey)
				);
			break;
			case APICommands::RequestRefund:
				$checksum = strtoupper(sha1($this->apiUser . $this->apiPassword . (string)APICommands::RequestRefund . $gatetransid . $amount . $this->apiKey));
			break;
			case APICommands::RequestCaptureAuth:
				$checksum = strtoupper(sha1($this->apiUser . $this->apiPassword . (string)APICommands::RequestCaptureAuth . $gatetransid . $this->apiKey));
			break;
			case APICommands::RequestCancelAuth:
				$checksum = strtoupper(sha1($this->apiUser . $this->apiPassword . (string)APICommands::RequestCancelAuth . $gatetransid . $this->apiKey));
			break;
			default:
				$this->throwException('Not implemented!', true, __LINE__);
			break;
		}

		return $checksum;
	}

	public function validateNotificationChecksum($xmldata){

		switch($xmldata->apiCmd){
			case 850:
				$chkstr = '850' . $xmldata->merchanttransid . $xmldata->amount . $xmldata->curcode;
			break;
			case 860:
				$chkstr = '860' . $xmldata->merchanttransid . $xmldata->amount . $xmldata->status;
			break;
			case 870:
				$chkstr = '870' . $xmldata->gatetransid . $xmldata->gaterebillid . $xmldata->status;
			break;
			case 880:
				$chkstr = '880' . $xmldata->gatetransid . $xmldata->status . $xmldata->amount;
			break;
			default:
			break;
		}

		return $xmldata->checksum == sha1($this->apiUser . $this->apiPassword . $chkstr . $this->apiKey);
	}

	function appendSimpleXMLElement(SimpleXMLElement $to, SimpleXMLElement $from){

		$toDom   = dom_import_simplexml($to);
		$fromDom = dom_import_simplexml($from);
		$toDom->appendChild($toDom->ownerDocument->importNode($fromDom, true));
	}

	function appendString($string, $keyword, $body){

		return substr_replace($string, PHP_EOL . $body, strpos($string, $keyword) + strlen($keyword), 0);
	}

	/**
	 * Round up and cast specified amount to float or string
	 *
	 * @param string|float $amount
	 * @param bool         $asFloat
	 *
	 * @return string|float
	 */
	public function formatAmount($amount, $asFloat = false){

		$amount = sprintf('%.2F', $amount); // "f" depends on locale, "F" doesn't
		return $asFloat ? (float)$amount : $amount;
	}

	public function getIpAddress(){

		if(!empty($_SERVER['HTTP_CLIENT_IP'])){   //check ip from share internet
			$ip = $_SERVER['HTTP_CLIENT_IP'];
		}
		elseif(!empty($_SERVER['HTTP_X_FORWARDED_FOR'])){ //to check ip is pass from proxy
			$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
		}
		else{
			$ip = $_SERVER['REMOTE_ADDR'];
		}

		return $ip;
	}

	function isSecure(){
		return (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off') || $_SERVER['SERVER_PORT'] == 443;
	}

	public function isValidXML($xml){
		$doc = @simplexml_load_string($xml);
		if($doc){
			return true; //this is valid
		}
		else{
			return false; //this is not valid
		}
	}

	public function getCountryIso($code){

		reset($this->iso_map);
		while(list($key, $value) = each($this->iso_map)){
			if($key == $code){
				return $value;
			}
		}
	}

	private function throwException($message, $die = false, $lineNumber = ''){

		$new_message = empty($lineNumber) ? $message : $message . ' GPNDataAPI line number: ' . $lineNumber;
		if($die){
			die($new_message);
		}
		else{
			throw new Exception($new_message);
		}
	}

	public function mapIso($code){

		$iso_map = [
				'AF' => 'AFG',
				'AX' => 'ALA',
				'AL' => 'ALB',
				'DZ' => 'DZA',
				'AS' => 'ASM',
				'AD' => 'AND',
				'AO' => 'AGO',
				'AI' => 'AIA',
				'AQ' => 'ATA',
				'AG' => 'ATG',
				'AR' => 'ARG',
				'AM' => 'ARM',
				'AW' => 'ABW',
				'AU' => 'AUS',
				'AT' => 'AUT',
				'AZ' => 'AZE',
				'BS' => 'BHS',
				'BH' => 'BHR',
				'BD' => 'BGD',
				'BB' => 'BRB',
				'BY' => 'BLR',
				'BE' => 'BEL',
				'BZ' => 'BLZ',
				'BJ' => 'BEN',
				'BM' => 'BMU',
				'BT' => 'BTN',
				'BO' => 'BOL',
				'BA' => 'BIH',
				'BW' => 'BWA',
				'BV' => 'BVT',
				'BR' => 'BRA',
				'IO' => 'IOT',
				'BN' => 'BRN',
				'BG' => 'BGR',
				'BF' => 'BFA',
				'BI' => 'BDI',
				'KH' => 'KHM',
				'CM' => 'CMR',
				'CA' => 'CAN',
				'CV' => 'CPV',
				'KY' => 'CYM',
				'CF' => 'CAF',
				'TD' => 'TCD',
				'CL' => 'CHL',
				'CN' => 'CHN',
				'CX' => 'CXR',
				'CC' => 'CCK',
				'CO' => 'COL',
				'KM' => 'COM',
				'CG' => 'COG',
				'CD' => 'COD',
				'CK' => 'COK',
				'CR' => 'CRI',
				'CI' => 'CIV',
				'HR' => 'HRV',
				'CU' => 'CUB',
				'CY' => 'CYP',
				'CZ' => 'CZE',
				'DK' => 'DNK',
				'DJ' => 'DJI',
				'DM' => 'DMA',
				'DO' => 'DOM',
				'EC' => 'ECU',
				'EG' => 'EGY',
				'SV' => 'SLV',
				'GQ' => 'GNQ',
				'ER' => 'ERI',
				'EE' => 'EST',
				'ET' => 'ETH',
				'FO' => 'FRO',
				'FK' => 'FLK',
				'FJ' => 'FJI',
				'FI' => 'FIN',
				'FR' => 'FRA',
				'GF' => 'GUF',
				'PF' => 'PYF',
				'TF' => 'ATF',
				'GA' => 'GAB',
				'GM' => 'GMB',
				'GE' => 'GEO',
				'DE' => 'DEU',
				'GH' => 'GHA',
				'GI' => 'GIB',
				'GB' => 'GBR',
				'GR' => 'GRC',
				'GL' => 'GRL',
				'GD' => 'GRD',
				'GP' => 'GLP',
				'GU' => 'GUM',
				'GT' => 'GTM',
				'GG' => 'GGY',
				'GN' => 'GIN',
				'GW' => 'GNB',
				'GY' => 'GUY',
				'HT' => 'HTI',
				'HM' => 'HMD',
				'HN' => 'HND',
				'HK' => 'HKG',
				'HU' => 'HUN',
				'IS' => 'ISL',
				'IN' => 'IND',
				'ID' => 'IDN',
				'IR' => 'IRN',
				'IQ' => 'IRQ',
				'IE' => 'IRL',
				'IM' => 'IMN',
				'IL' => 'ISR',
				'IT' => 'ITA',
				'JM' => 'JAM',
				'JP' => 'JPN',
				'JO' => 'JOR',
				'KZ' => 'KAZ',
				'KE' => 'KEN',
				'KI' => 'KIR',
				'KP' => 'PRK',
				'KR' => 'KOR',
				'KW' => 'KWT',
				'KG' => 'KGZ',
				'LA' => 'LAO',
				'LV' => 'LVA',
				'LB' => 'LBN',
				'LS' => 'LSO',
				'LR' => 'LBR',
				'LY' => 'LBY',
				'LI' => 'LIE',
				'LT' => 'LTU',
				'LU' => 'LUX',
				'MO' => 'MAC',
				'MK' => 'MKD',
				'MG' => 'MDG',
				'MW' => 'MWI',
				'MY' => 'MYS',
				'MV' => 'MDV',
				'ML' => 'MLI',
				'MT' => 'MLT',
				'MH' => 'MHL',
				'MQ' => 'MTQ',
				'MR' => 'MRT',
				'MU' => 'MUS',
				'YT' => 'MYT',
				'MX' => 'MEX',
				'FM' => 'FSM',
				'MD' => 'MDA',
				'MC' => 'MCO',
				'MN' => 'MNG',
				'ME' => 'MNE',
				'MS' => 'MSR',
				'MA' => 'MAR',
				'MZ' => 'MOZ',
				'MM' => 'MMR',
				'NA' => 'NAM',
				'NR' => 'NRU',
				'NP' => 'NPL',
				'NL' => 'NLD',
				'AN' => 'ANT',
				'NC' => 'NCL',
				'NZ' => 'NZL',
				'NI' => 'NIC',
				'NE' => 'NER',
				'NG' => 'NGA',
				'NU' => 'NIU',
				'NF' => 'NFK',
				'MP' => 'MNP',
				'NO' => 'NOR',
				'OM' => 'OMN',
				'PK' => 'PAK',
				'PW' => 'PLW',
				'PS' => 'PSE',
				'PA' => 'PAN',
				'PG' => 'PNG',
				'PY' => 'PRY',
				'PE' => 'PER',
				'PH' => 'PHL',
				'PN' => 'PCN',
				'PL' => 'POL',
				'PT' => 'PRT',
				'PR' => 'PRI',
				'QA' => 'QAT',
				'RE' => 'REU',
				'RO' => 'ROU',
				'RU' => 'RUS',
				'RW' => 'RWA',
				'BL' => 'BLM',
				'SH' => 'SHN',
				'KN' => 'KNA',
				'LC' => 'LCA',
				'MF' => 'MAF',
				'PM' => 'SPM',
				'VC' => 'VCT',
				'WS' => 'WSM',
				'SM' => 'SMR',
				'ST' => 'STP',
				'SA' => 'SAU',
				'SN' => 'SEN',
				'RS' => 'SRB',
				'SC' => 'SYC',
				'SL' => 'SLE',
				'SG' => 'SGP',
				'SK' => 'SVK',
				'SI' => 'SVN',
				'SB' => 'SLB',
				'SO' => 'SOM',
				'ZA' => 'ZAF',
				'GS' => 'SGS',
				'ES' => 'ESP',
				'LK' => 'LKA',
				'SD' => 'SDN',
				'SR' => 'SUR',
				'SJ' => 'SJM',
				'SZ' => 'SWZ',
				'SE' => 'SWE',
				'CH' => 'CHE',
				'SY' => 'SYR',
				'TW' => 'TWN',
				'TJ' => 'TJK',
				'TZ' => 'TZA',
				'TH' => 'THA',
				'TL' => 'TLS',
				'TG' => 'TGO',
				'TK' => 'TKL',
				'TO' => 'TON',
				'TT' => 'TTO',
				'TN' => 'TUN',
				'TR' => 'TUR',
				'TM' => 'TKM',
				'TC' => 'TCA',
				'TV' => 'TUV',
				'UG' => 'UGA',
				'UA' => 'UKR',
				'AE' => 'ARE',
				'US' => 'USA',
				'UM' => 'UMI',
				'UY' => 'URY',
				'UZ' => 'UZB',
				'VU' => 'VUT',
				'VA' => 'VAT',
				'VE' => 'VEN',
				'VN' => 'VNM',
				'VG' => 'VGB',
				'VI' => 'VIR',
				'WF' => 'WLF',
				'EH' => 'ESH',
				'YE' => 'YEM',
				'ZM' => 'ZMB',
				'ZW' => 'ZWE',
		];

		try {
			if(array_key_exists($code, $iso_map)){
				return $iso_map[$code];
			}
			else{
				return false;
			}
		}
		catch(Exception $exception){
			$this->throwException($exception->getMessage(), true);
		}
	}

	//PROPERTIES
	/** @var bool - If debug_mode error messages will be shown/dumped. */
	private $debug_mode;

	/** @var string apiKey */
	private $apiUser;

	/** @var string apiPassword */
	private $apiPassword;

	/** @var string apiKey */
	private $apiKey;

	/** @var string transactionUrl - this is the full API url */
	public $transactionUrl;

	/** @var string paymentPageUrl - this is the payment page url */
	public $paymentPageUrl;

	/** @var string successUrl - used in 710 - PaymentPage */
	public $successUrl;

	/** @var string successUrl - used in 710 - PaymentPage */
	public $failureUrl;

	/** @var string successUrl - used in 710 - PaymentPage */
	public $contactUrl;

	/** @var string logoUrl - The full URL to the image which you wish to be displayed as your logo on Payment Page. Must be 200px width. - PaymentPage */
	public $logoUrl;
	
	/** @var string prettyName - this is the prettyName */
	public $prettyName;
	
	/** @var string iframeScript - this is the full iFrame Script Url - iFrame */
	public $iframeUrl;
	
	/** @var string merchantName - Your Merchant name as you wish it to appear on the order - PaymentPage */
	public $merchantName;

	/** @var xmlValidator class */
	public $validator;
}

class APIRequest
{

	public function __construct(){
	}

	public $response;

	public function executeRequest($requestData, $url){

		$http = curl_init();

		//Set up curl parameters
		curl_setopt($http, CURLOPT_URL, $url);    // set remote address
		curl_setopt($http, CURLOPT_POSTFIELDS, http_build_query(['strrequest' => $requestData]));
		curl_setopt($http, CURLOPT_RETURNTRANSFER, 1);  // Make CURL pass the response as a curl_exec return value instead of outputting to STDOUT
		curl_setopt($http, CURLOPT_POST, 1);        // Activate the POST method
		curl_setopt($http, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($http, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($http, CURLOPT_VERBOSE, 0);
		curl_setopt($http, CURLOPT_HEADER, 0);
		curl_setopt($http, CURLOPT_SSLVERSION, 6);
		//execute the connection
		$result = curl_exec($http);

		$this->response = $this->getResponse($http, $result);

		curl_close($http);

	}

	private function getHttpStatusCode($curlResponse){

		return curl_getinfo($curlResponse, CURLINFO_HTTP_CODE);
	}

	public function getResponse($curlChannel, $result){

		$xml_out     = null;
		$http_status = $this->getHttpStatusCode($curlChannel);
		$xml_out     = simplexml_load_string($result);

		if(!$xml_out || $http_status != '200'){

			if($http_status == 0){
				$err     = curl_error($curlChannel);
				$xml_out = simplexml_load_string(
						'<?xml version="1.0" encoding="utf-8"?><dummyresponse><result>ERROR</result><errorcode>001</errorcode><errormessage>Dummy response - CURL returned error - ' .
						$err .
						'</errormessage></dummyresponse>'
				);
			}
			else{
				$xml_out = simplexml_load_string(
						'<?xml version="1.0" encoding="utf-8"?><dummyresponse><result>ERROR</result><errorcode>001</errorcode><errormessage>Dummy response -> Internal error - response from payment gateway malformed or indicates error</errormessage></dummyresponse>'
				);
			}
		}
		elseif($xml_out->result != TransactionStatus::SUCCESS){
			$xml_out = simplexml_load_string($xml_out->asXML());
		}

		return $xml_out;
	}

	public function getTransactionResult(){
		return $this->getResponseValue('result');
	}

	public function getMerchantTransId(){
		return $this->getResponseValue('merchanttransid');
	}

	public function getErrorCode(){
		return $this->getResponseValue('errorcode');
	}

	public function getErrorMsg(){
		return $this->getResponseValue('errormessage');
	}

	public function getDescription(){
		return $this->getResponseValue('description');
	}


	public function getGateTransactionId(){
		return $this->getResponseValue('gatetransid');
	}

	public function isNotError(){
		$re = $this->getResponseValue('result');

		return isset($re) && $re != TransactionStatus::ERROR;
	}

	public function getResponseValue($key, $default = null){

		if(!$this->response){
			$this->throwException('Response XML is not valid', true);
		}

		return isset($this->response->$key) ? (string)$this->response->$key : $default;
	}

}

class xmlValidator
{
	public function __construct(){
		$this->validationFieldsCount = count($this->_API_XML_FIELDS_LEN);
	}

	const Required     = 'REQ';
	const Optional     = 'OPT';
	const MustBeNumber = 'NUM';


	const TransactionElement = 'TRANS';
	const CustomerElement    = 'CUST';
	const ProductElement     = 'PROD';

	/** @var int - Contains count of validation fields */
	public $validationFieldsCount;

	private $_API_XML_FIELDS_LEN = [
			['merchanttransid', xmlValidator::Required, '50'],
			['amount', xmlValidator::Required, '16'],
			['description', xmlValidator::Required, '100'],
			['merchantspecific1', xmlValidator::Optional, '200'],
			['merchantspecific2', xmlValidator::Optional, '200'],
			['merchantspecific3', xmlValidator::Optional, '200'],
			['merchantspecific4', xmlValidator::Optional, '200'],
			['merchantspecific5', xmlValidator::Optional, '200'],

			['firstname', xmlValidator::Required, '50', xmlValidator::CustomerElement],
			['lastname', xmlValidator::Required, '50', xmlValidator::CustomerElement],
			['email', xmlValidator::Required, '100', xmlValidator::CustomerElement],
			['countryiso', xmlValidator::Optional, '3', xmlValidator::CustomerElement],
			['stateregioniso', xmlValidator::Optional, '2', xmlValidator::CustomerElement],
			['zippostal', xmlValidator::Optional, '15', xmlValidator::CustomerElement],
			['city', xmlValidator::Optional, '50', xmlValidator::CustomerElement],
			['address1', xmlValidator::Optional, '75', xmlValidator::CustomerElement],
			['address2', xmlValidator::Optional, '75', xmlValidator::CustomerElement],
			['phone1phone', xmlValidator::Optional, '10', xmlValidator::CustomerElement],
			['accountid', xmlValidator::Optional, '50', xmlValidator::CustomerElement],
			['ipaddress', xmlValidator::Required, '15', xmlValidator::CustomerElement],

			['name', xmlValidator::Required, '50', xmlValidator::ProductElement],
			['description', xmlValidator::Required, '50', xmlValidator::ProductElement],
			['price', xmlValidator::Required, '16', xmlValidator::ProductElement],
			['amount', xmlValidator::Required, '8', xmlValidator::ProductElement]
	];

	public function validateXmlFields($validationObject, $debug = false){

		$validationErrors = '';

		try {
			$xml = $validationObject->asXML();
			if($this->isXMLContentValid($xml)){
				$childrenCount = count($validationObject->children());
				for($x = 0; $x < $this->validationFieldsCount; $x++){
					$validationField = $this->_API_XML_FIELDS_LEN[$x];
					if($validationField[1] == xmlValidator::Required){
						$fields = $validationObject->children();
						for($x2 = 0; $x2 < $childrenCount; $x2++){
							$field = $fields[$x2];
							if($field->getName() == $validationField[0]){
								//start the string len validation
								if(strlen($field) == 0){
									$validationErrors = $this->addValidationError($validationErrors, $validationField[0], 'FIELD_IS_REQUIRED', 'This field is required!');
								}
								if(strlen($field) > (int)$validationField[2]){
									$validationErrors = $this->addValidationError($validationErrors, $validationField[0], 'LEN_OF_FIELD_TO_LONG', 'Len of field to long!');
									echo $validationField[0];
								}

								break; //we have done with this field validation..
							}
						}
					}
				}
			}
			else{
				//	$validationResult = array_push('XML VALIDATION ERROR!');
			}
		}
		catch(Exception $e){
			echo 'Caught exception: ', $e->getMessage(), "\n";
		}
		if(!$debug){
			return (strlen($validationErrors) == 0);
		}
		else{

			$validationResultXML = new SimpleXMLElement('<validation></validation>');
			$validationResultXML->addChild('errors', $validationErrors);

			$errCount = count($validationResultXML->errors->children());
			if($errCount > 0){
				print_r($validationResultXML->errors->children());
			}

			return $errCount == 0 ? true : false;
		}

	}

	private function addValidationError(&$errors, $xml_tag, $type, $message){

		$new_error = '	<error>
							<xml_tag>' . $xml_tag . '</xml_tag>
							<type>' . $type . '</type>
							<message>' . $message . '</message>
						</error>';

		return $errors .= $new_error;
	}

	/**
	 * @param string $xmlContent A well-formed XML string
	 * @param string $version    1.0
	 * @param string $encoding   utf-8
	 *
	 * @return bool
	 */
	public function isXMLContentValid($xmlContent, $version = '1.0', $encoding = 'utf-8'){

		if(trim($xmlContent) == ''){
			return false;
		}

		libxml_use_internal_errors(true);
		$doc = new DOMDocument($version, $encoding);
		$doc->loadXML($xmlContent);

		$errors = libxml_get_errors();
		libxml_clear_errors();

		return empty($errors);
	}

}